/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.util.cachemap;

import java.util.*;

class CacheMapEntry implements Map.Entry
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Object key;
  private Object value;
  private int hits = 0;
  private long timestamp;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  CacheMapEntry(Object key, Object value)
  {
    this.key = key;
    this.value = value;
    this.timestamp = System.currentTimeMillis();

  } // end constructor

  /*--------------------------------------------------------------------------------
   * finalize() function
   *--------------------------------------------------------------------------------
   */

  protected void finalize()
  {
    key = null;
    value = null;

  } // end finalize

  /*--------------------------------------------------------------------------------
   * Implementations from interface MapEntry
   *--------------------------------------------------------------------------------
   */

  public final Object getKey()
  {
    return key;

  } // end getKey

  public final Object getValue()
  {
    return value;

  } // end getValue

  public final Object setValue(Object o)
  {
    Object rc = value;
    value = o;
    return rc;

  } // end setValue

  public final boolean equals(Object o)
  {
    // make sure the other element is a Map.Entry
    if ((o==null) || !(o instanceof Map.Entry))
      return false;
    Map.Entry other = (Map.Entry)o;

    // compare the keys
    if (key==null)
    { // the other key must be null
      if (other.getKey()!=null)
	return false;

    } // end if
    else
    { // the other key must be equal to us
      if ((other.getKey()==null) || (!(key.equals(other.getKey()))))
	return false;

    } // end else

    // compare the values
    if (value==null)
      return (other.getValue()==null);
    else
      return ((other.getValue()!=null) && value.equals(other.getValue()));

  } // end equals

  public final int hashCode()
  {
    int rc = 0;
    if (key!=null)
      rc ^= key.hashCode();
    if (value!=null)
      rc ^= value.hashCode();
    return rc;

  } // end hashCode

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  final int getHits()
  {
    return hits;

  } // end getHits

  final long getTimestamp()
  {
    return timestamp;

  } // end getTimestamp

  final long getAge(long tick)
  {
    return (tick - timestamp);

  } // end getAge

  final void touch()
  {
    hits++;
    timestamp = System.currentTimeMillis();

  } // end touch

} // end class CacheMapEntry
