/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.security.AuditRecord;

class AdminOperationsImpl implements AdminOperations
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(AdminOperationsImpl.class.getName());

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private EngineBackend engine;            // the back end of the engine
  private UserBackend user;                // the UserContext that created this object
  private DataPool datapool;               // the data pool used by this object

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  AdminOperationsImpl(EngineBackend engine, UserBackend user, DataPool datapool)
  {
    this.engine = engine;
    this.user = user;
    this.datapool = datapool;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface AdminOperations
   *--------------------------------------------------------------------------------
   */

  public List getAuditRecords(int offset, int count) throws DataException
  {
    Connection conn = null;
    List rc = null;

    try
    { // retrieve a connection from the data pool and get the audit records
      conn = datapool.getConnection();
      rc = AuditRecord.getAuditRecords(conn,-1,offset,count);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("error loading audit records: " + e.getMessage(),e);
      throw new DataException("unable to load audit records: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return rc;

  } // end getAuditRecords

  public int getAuditRecordCount() throws DataException
  {
    Connection conn = null;
    int rc = -1;

    try
    { // retrieve a connection from the data pool and get the audit records
      conn = datapool.getConnection();
      rc = AuditRecord.getAuditRecordCount(conn,-1);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("error loading audit record count: " + e.getMessage(),e);
      throw new DataException("unable to load audit record count: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return rc;

  } // end getAuditRecordCount

} // end class AdminOperationsImpl
